# Copyright (c) 2010 gocept gmbh & co. kg
# See also LICENSE.txt

import cookielib
import urllib
import urllib2
import os
import re
import logging

log = logging.getLogger(__name__)


MATCH_NEW_TICKET_NO = re.compile(r'Ticket\s*(\d+)')


class Ticket(dict):

    def __init__(self, rt):
        super(Ticket, self).__init__()
        self.rt = rt

    @classmethod
    def fromResponse(cls, response, rt):
        ticket = Ticket(rt)
        response = response[:]
        assert "RT/3.4.3 200 Ok\n" == response.pop(0)
        assert "\n" == response.pop(0)
        assert "\n" == response.pop()
        for line in response:
            if line == "\n":
                continue
            if ':' in line:
                key, value = line.split(':', 1)
                ticket[key] = value.strip()
            else:
                if not isinstance(ticket[key], list):
                    ticket[key] = [ticket[key]]
                ticket[key].append(line.strip())
        ticket['id'] = ticket['id'].replace('ticket/', '')
        return ticket

    def edit(self, data):
        self.rt.edit_ticket(self['id'], data)


class RT(object):

    DATE_FORMAT = '%a %b %d %H:%M:%S %Y'

    def __init__(self, user, password, url):
        self.cookies = cookielib.LWPCookieJar()
        self.opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(self.cookies))
        self.url = '%s/REST/1.0' % url
        self.user = user
        self.password = password
        self.login()

    def open(self, url, data=None):
        if data is not None:
            for key, value in data.items():
                if isinstance(value, unicode):
                    data[key] = value.encode('utf8')
            data = urllib.urlencode(data)
        return self.opener.open(url, data)

    def login(self):
        response = self.open(self.url, {'user': self.user, 'pass': self.password})
        response.read()

    def show_ticket(self, ticket_id):
        data = {}
        response = self.open(self.url+'/ticket/%s/show' % ticket_id)
        return Ticket.fromResponse(response.readlines(), self)

    def search(self, query):
        query = urllib.quote(query)
        tickets = []
        response = self.open(self.url+'/search/ticket?format=i&query=%s' % query)
        lines = response.readlines()
        assert "RT/3.4.3 200 Ok\n" == lines.pop(0)
        assert "\n" == lines.pop(0)
        for line in lines:
            line = line.strip()
            prefix, ticket = line.split('/')
            assert prefix == 'ticket'
            tickets.append(self.show_ticket(ticket))
        return tickets

    def new_ticket(self, content):
        content['id'] = 'ticket/new'
        if 'Text' in content:
            content['Text'] = content['Text'].replace('\n', '\n  ')
        content_list = []
        for key, value in content.items():
            if isinstance(value, list):
                for val in value:
                    content_list.append('%s: %s' % (key, val))
            else:
                content_list.append('%s: %s' % (key, value))

        data = dict(content='\n'.join(content_list))
        r = self.open(self.url+'/ticket/new', data)
        result = r.read()
        m = MATCH_NEW_TICKET_NO.search(result)
        try:
            ticket = m.groups()[0]
        except KeyError:
            raise RuntimeError(result)
        return self.show_ticket(ticket)

    def edit_ticket(self, ticket_id, data):
        content = dict(content='\n'.join('%s: %s' % (key, value)
                                         for key, value in data.items()))
        r = self.open(self.url+'/ticket/%s/edit' % ticket_id, content)
        r.read()
